@@ -1,12 +1,14 @@
 import os
+import time
 
 NAME = "MediaBox"
 OSSO_NAME = "de.pycage.mediabox"
-VERSION = "0.96.1"
+VERSION = "0.96.2"
 
 AUTHORS = ["Martin Grimme  <martin.grimme@lintegra.de>"]
-COPYRIGHT = "\xc2\xa9 2007 - 2008 Martin Grimme"
+COPYRIGHT = "\xc2\xa9 2007 - 2009 Martin Grimme"
 
 USER_DIR = os.path.expanduser("~/.mediabox")
 MEDIABOX_DIR = os.path.join(os.path.dirname(__file__), "..")
 
+START_TIME = time.time()
